@@ -974,7 +974,7 @@ f_starttls( SNET *sn, int ac, char **av )
         return( 1 );
     }
 
-    if ( authlevel == 2 ) {
+    if ( authlevel >= 2 ) {
 	if (( peer = SSL_get_peer_certificate( sn->sn_ssl ))
 		== NULL ) {
 	    syslog( LOG_ERR, "no peer certificate" );
@@ -1092,7 +1092,7 @@ f_login( SNET *sn, int ac, char **av )
     int				retval;
     pam_handle_t		*pamh;
     struct pam_conv		pam_conv = {
-	exchange,
+	(int (*)())exchange,
 	NULL
     };
 
