@@ -32,6 +32,7 @@ int		dodots = 0;
 int		dotfd;
 int		lastpercent = -1;
 int		case_sensitive = 1;
+int		tran_format = -1; 
 extern int	exclude_warnings;
 const EVP_MD    *md;
 
@@ -258,7 +259,7 @@ main( int argc, char **argv )
     int 		errflag = 0, use_outfile = 0;
     int			finish = 0;
     struct stat		st;
-    char		type;
+    char		type, buf[ MAXPATHLEN ];
     struct applefileinfo	afinfo;
 
     edit_path = CREATABLE;
@@ -353,10 +354,51 @@ main( int argc, char **argv )
     }
 
     path_prefix = argv[ optind ];
-    /* Clip trailing '/' */
     len = strlen( path_prefix );
+
+    /* Clip trailing '/' */
     if (( len > 1 ) && ( path_prefix[ len - 1 ] == '/' )) {
-	path_prefix[ len - 1] = '\0';
+	path_prefix[ len - 1 ] = '\0';
+	len--;
+    }
+
+    /* If path_prefix doesn't contain a directory, canonicalize it by
+     * prepending a "./".  This allow paths to be dynamically converted between
+     * relative and absolute paths without breaking sort order.
+     */
+    switch( path_prefix[ 0 ] ) {
+    case '/':
+        break;
+
+    case '.':
+	/* Don't rewrite '.' or paths starting with './' */
+	if (( len == 1 ) || (  path_prefix[ 1 ] == '/' )) {
+	    break;
+	}
+    default:
+        if ( snprintf( buf, sizeof( buf ), "./%s",
+                path_prefix ) >= MAXPATHLEN ) {
+            fprintf( stderr, "path too long\n" );
+            exit( 2 );
+        }
+	path_prefix = buf;
+        break;
+    }
+
+    /* Determine if called with relative or absolute pathing.  Path is relative
+     * if it's just '.' or starts with './'.  File names that start with a '.'
+     * are absolute.
+     */
+    if ( path_prefix[ 0 ] == '.' ) {
+	if ( len == 1 ) {
+	    tran_format = T_RELATIVE;
+	} else if ( path_prefix[ 1 ] == '/' ) {
+	    tran_format = T_RELATIVE;
+	} else {
+	    tran_format = T_ABSOLUTE;
+	}
+    } else {
+	tran_format = T_ABSOLUTE;
     }
 
     if ( radstat( path_prefix, &st, &type, &afinfo ) != 0 ) {
