@@ -277,6 +277,12 @@ do_lcksum( char *tpath )
 	if ( *targv[ 0 ] == '-' ) {
 	    remove = 1;
 	    targv++;
+	    tac--;
+
+	    if ( tac == 1 ) {
+		fprintf( stderr, "line %d: invalid transcript line\n", linenum);
+		goto badline;
+	    }
 	} else {
 	    remove = 0;
 	}
@@ -308,8 +314,8 @@ do_lcksum( char *tpath )
 	bytes = 0;
 
 	/* Check transcript order */
-	if ( prepath != 0 ) {
-	    if ( pathcasecmp( path, prepath, case_sensitive ) < 0 ) {
+	if ( *prepath != '\0' ) {
+	    if ( pathcasecmp( path, prepath, case_sensitive ) <= 0 ) {
 		fprintf( stderr, "line %d: bad sort order\n", linenum );
 		cleanup( updatetran, upath );
 		exit( 2 );
