@@ -164,7 +164,7 @@ transcript_parse( struct transcript *tran )
 	exit( 2 );
     }
 
-    if ( pathcasecmp( epath, tran->t_pinfo.pi_name, case_sensitive ) < 0 ) {
+    if ( pathcasecmp( epath, tran->t_pinfo.pi_name, case_sensitive ) <= 0 ) {
 	fprintf( stderr, "%s: line %d: bad sort order\n",
 	    tran->t_fullname, tran->t_linenum );
 	exit( 2 );
@@ -253,6 +253,11 @@ transcript_parse( struct transcript *tran )
 		tran->t_fullname, tran->t_linenum );
 	    exit( 2 );
 	}
+	if (( epath = convert_path_type( epath )) == NULL ) {
+	    fprintf( stderr, "%s: line %d: path too long\n",
+		    tran->t_fullname, tran->t_linenum );
+	    exit( 2 );
+	}
 	strcpy( tran->t_pinfo.pi_link, epath );
 	break;
 
@@ -624,11 +629,11 @@ t_compare( struct pathinfo *fs, struct transcript *tran )
 	if ( tran->t_type != T_NEGATIVE ) {
 	    if (( fs->pi_stat.st_uid != tran->t_pinfo.pi_stat.st_uid ) ||
 		    ( fs->pi_stat.st_gid != tran->t_pinfo.pi_stat.st_gid ) || 
+		    ( dev != tran->t_pinfo.pi_stat.st_rdev ) ||
 		    ( mode != tran_mode )) {
 		t_print( fs, tran, PR_STATUS );
 	    }
-	}
-	if ( dev != tran->t_pinfo.pi_stat.st_rdev ) {
+	} else if ( dev != tran->t_pinfo.pi_stat.st_rdev ) {
 	    t_print( fs, tran, PR_STATUS );
 	}	
 	break;
