@@ -23,6 +23,7 @@
 const EVP_MD    *md;
 
 int		case_sensitive = 1;
+int		tran_format = -1; 
 
 /*
  * exit codes:
@@ -37,26 +38,9 @@ twhich( char *pattern, int displayall )
     struct node		*node;
     struct transcript	*tran;
     extern struct transcript	*tran_head;
-    extern struct list	*special_list;
     extern struct list	*exclude_list;
     int			cmp = 0, match = 0;
 
-    /* check special list */
-    if ( special_list->l_count > 0 ) {
-	for ( node = list_pop_head( special_list ); node != NULL;
-		node = list_pop_head( special_list )) {
-	    if ( pathcasecmp( node->n_path, pattern, case_sensitive ) == 0 ) {
-		printf( "# Special\n" );
-		printf( "special.T:\n" );
-		printf( "%s\n", node->n_path );
-		free( node );
-		if ( !displayall ) {
-		    goto done;
-		}
-	    }
-	}
-    }
-
     /* check exclude list */
     if ( exclude_list->l_count > 0 ) {
 	for ( node = list_pop_head( exclude_list ); node != NULL;
@@ -190,9 +174,9 @@ main( int argc, char **argv )
     }
 
     if ( err ) {
-        fprintf( stderr, "Usage: %s [ -aIV ] [ -K command file ] file\n",
+        fprintf( stderr, "Usage: %s [ -aIrV ] [ -K command file ] file\n",
 	    argv[ 0 ] );
-        fprintf( stderr, "Usage: %s -s -K command [ -aIV ] file\n",
+        fprintf( stderr, "Usage: %s -s -K command [ -aIrV ] file\n",
 	    argv[ 0 ] );
         exit( 2 );
     }
@@ -200,6 +184,21 @@ main( int argc, char **argv )
     /* clip trailing slash */
     if ( len > 1 && pattern[ len - 1 ] == '/' ) {
 	pattern[ len - 1 ] = '\0';
+	len--;
+    }
+
+    /* Determine if called with relative or absolute pathing.  Path is relative
+     * if it's just '.' or starts with './'.  File names that start with a '.'
+     * are absolute.
+     */
+    if ( pattern[ 0 ] == '.' ) {
+	if ( len == 1 ) {
+	    tran_format = T_RELATIVE;
+	} else if ( pattern[ 1 ] == '/' ) {
+	    tran_format = T_RELATIVE;
+	}
+    } else {
+	tran_format = T_ABSOLUTE;
     }
 
     /* initialize the transcripts */
